<?php

namespace Drupal\widget_ingestion\Plugin\QueueWorker;

use Composer\Semver\Comparator;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\widget_type\Entity\WidgetType;
use Drupal\widget_type\WidgetTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Create/update a widget entity.
 *
 * @QueueWorker(
 *   id = "widget_ingestion_queue",
 *   title = @Translation("Widget queue worker"),
 *   cron = {"time" = 3}
 * )
 */
final class WidgetQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The cache tags invalidator service.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * ReportWorkerBase constructor.
   *
   * @param array $configuration
   *   The configuration of the instance.
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cache_tags_invalidator
   *   The cache tags invalidator service.
   *
   * @SuppressWarnings(PHPMD.LongVariable)
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, CacheTagsInvalidatorInterface $cache_tags_invalidator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->cacheTagsInvalidator = $cache_tags_invalidator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('cache_tags.invalidator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $this->updateEntity($data);
  }

  /**
   * Save the widget entity, populated by the correct data.
   *
   * @param array $data
   *   Widget data received from the widget registry.
   *
   * @return \Drupal\widget_type\WidgetTypeInterface
   *   New widget if it does not already exist, or the existing widget.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Core\Entity\EntityStorageException
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  private function updateEntity(array $data): WidgetTypeInterface {
    $ids = $this->entityTypeManager
      ->getStorage('widget_type')
      ->getQuery()
      ->condition('remote_widget_id', $data['shortcode'])
      ->range(0, 1)
      ->execute();

    $widget_is_new = (bool) $ids;
    $widget_type = $widget_is_new
      ? WidgetType::load(current($ids))
      : WidgetType::create(['remote_widget_id' => $data['shortcode']]);
    assert($widget_type instanceof WidgetTypeInterface);
    if ($ids && Comparator::lessThanOrEqualTo($data['version'], $widget_type->getVersion())) {
      // Nothing to do.
      return $widget_type;
    }
    $this->importWidget($widget_type, $data);

    // When importing a new widget, invalidate library info cache.
    if ($widget_is_new) {
      $this->cacheTagsInvalidator->invalidateTags(['library_info']);
    }
    return $widget_type;
  }

  /**
   * Imports the JS widget.
   *
   * @param \Drupal\widget_type\WidgetTypeInterface $widget_type
   *   The entity being saved.
   * @param array $data
   *   Structured array with widget data.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function importWidget(WidgetTypeInterface $widget_type, array $data): void {
    // Set values from $data, except for the short code which remains invariant.
    $widget_type
      ->setVersion($data['version'])
      ->setDirectory($data['directoryUrl'])
      ->setName($data['title'])
      ->setDescription($data['description'] ?? '')
      ->setSettingsSchema($this->extractSettings($data['settingsSchema'] ?? []))
      ->setFiles($data['files'] ?? []);
    $avail_trans = $data['availableTranslations'] ?? NULL;
    if ($avail_trans) {
      $widget_type->setRemoteLanguages($avail_trans);
    }
    $widget_type->save();
  }

  /**
   * Extracts the settings from the widget as a PHP array.
   *
   * @param array $settings
   *   Widget settings schema in JSON Schema format.
   *
   * @return array
   *   An array of widget settings.
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  private function extractSettings(array $settings): array {
    return NestedArray::getValue(
      $settings,
      ['properties', 'fields']
    ) ?: [];
  }

}
