<?php

namespace Drupal\widget_ingestion;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Queue\QueueWorkerManagerInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Service to interact with the widget registry.
 */
final class WidgetRemoteRepository {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  private $client;

  /**
   * The widget registry endpoint.
   *
   * @var string
   */
  private $endpoint = '';

  /**
   * The queue worker.
   *
   * @var \Drupal\Core\Queue\QueueWorkerInterface
   */
  private $queueWorker;

  /**
   * WidgetRemoteRepository constructor.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The guzzle client to make HTTP requests.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Queue\QueueWorkerManagerInterface $queue_worker_manager
   *   The queue worker manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function __construct(ClientInterface $client, ConfigFactoryInterface $config_factory, QueueWorkerManagerInterface $queue_worker_manager) {
    $this->client = $client;
    $settings = $config_factory->get('widget_ingestion.settings');
    $this->endpoint = $settings->get('endpoint');
    $this->queueWorker = $queue_worker_manager
      ->createInstance('widget_ingestion_queue');
  }

  /**
   * Gets a list of all widgets from the widget registry.
   *
   * @return array
   *   The list of all widgets.
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function listAll() {
    try {
      $response = $this->client->get($this->endpoint, [
        'headers' => [
          'Accept' => 'application/json',
        ],
      ]);
      return Json::decode($response->getBody());
    }
    catch (GuzzleException $e) {
      watchdog_exception('widget_ingestion', $e);
      return [];
    }
  }

  /**
   * Re-ingest all the widgets.
   */
  public function ingestAll() {
    $items = $this->listAll();
    array_map([$this->queueWorker, 'processItem'], $items);
    return $items;
  }

}
